package data;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Scanner;

public class EigenvalueWeightedPcGrmReader implements IGrmReader{
//	private static final int MAX_VECS = 2;
	private double[][] m_scaled_vecs;
	private IndividualVariables m_ids;
	private int m_index1 = -1;
	private int m_index2 = -1;
	private boolean m_has_next = true;
	
	public EigenvalueWeightedPcGrmReader(IndividualVariables ids, String vec_file, String val_file) throws FileNotFoundException {
		IndividualVariables vecs = IndividualVariables.read(vec_file);
		if( ids == null )
		{
			m_ids = vecs;
		}
		else
		{
			m_ids = ids;
		}
		List<IndividualId> ordered = m_ids.getOrderedIndividualIds();
		double[] vals = readScaledVals(val_file);
		int variable_count = vecs.getVariableCount();
//		int variable_count = MAX_VECS;
		m_scaled_vecs = new double[variable_count][];
		for(int vec_index = 0; vec_index < variable_count; vec_index++)
		{
			double[] array = vecs.getScaledArray(ordered, vec_index);
			double scale = Math.sqrt(vals[vec_index]);
			for(int individual_index = 0; individual_index < array.length; individual_index++)
			{
				array[individual_index] *= scale;
			}
			m_scaled_vecs[vec_index] = array;
		}
	}

	private static double[] readScaledVals(String val_file) throws FileNotFoundException {
		List<Double> vals = new ArrayList<Double>();
		Scanner scanner = new Scanner(new BufferedInputStream(new FileInputStream(val_file)));
		while(scanner.hasNext())
		{
			vals.add(scanner.nextDouble());
		}
		double val_sum = 0;
		double[] val_array = new double[vals.size()];
		for(int i = 0; i < vals.size(); i++)
		{
			val_array[i] = vals.get(i);
			val_sum += val_array[i];
		}
		for(int i = 0; i < val_array.length; i++)
		{
			val_array[i] /= val_sum;
		}
		return val_array;
	}

	@Override
	public float getValue() {
		float value = 0;
		for(int vec_index = 0; vec_index < m_scaled_vecs.length; vec_index++)
		{
			value += m_scaled_vecs[vec_index][m_index1]*m_scaled_vecs[vec_index][m_index2];
		}
		return value;
	}

	@Override
	public int getIndex1() {
		return m_index1;
	}

	@Override
	public int getIndex2() {
		return m_index2;
	}

	@Override
	public void next() {
		if( !m_has_next  )
		{
			throw new NoSuchElementException();
		}
		if( m_index2 == m_index1 )
		{
			m_index2 = 0;
			m_index1++;
		}
		else
		{
			m_index2++;
		}
		if( m_index2 == (m_scaled_vecs[0].length - 1) )
		{
			m_has_next = false;
		}
	}

	@Override
	public boolean hasNext() {
		return m_has_next;
	}

	@Override
	public IndividualVariables getIds() {
		return m_ids;
	}
	
	

}
