package data;

import java.io.FileNotFoundException;
import java.util.List;
import java.util.NoSuchElementException;

public class SinglePcGrmReader implements IGrmReader{
	private double[] m_scaled_vec;
	private IndividualVariables m_ids;
	private int m_index1 = -1;
	private int m_index2 = -1;
	private boolean m_has_next = true;
	
	public SinglePcGrmReader(IndividualVariables ids, IndividualVariables vectors, int vector_index) throws FileNotFoundException {
		if( ids == null )
		{
			m_ids = vectors;
		}
		else
		{
			m_ids = ids;
		}
		List<IndividualId> ordered = m_ids.getOrderedIndividualIds();
		m_scaled_vec = vectors.getScaledArray(ordered, vector_index);
	}

	@Override
	public float getValue() {
		return (float)(m_scaled_vec[m_index1]*m_scaled_vec[m_index2]);
	}

	@Override
	public int getIndex1() {
		return m_index1;
	}

	@Override
	public int getIndex2() {
		return m_index2;
	}

	@Override
	public void next() {
		if( !m_has_next  )
		{
			throw new NoSuchElementException();
		}
		if( m_index2 == m_index1 )
		{
			m_index2 = 0;
			m_index1++;
		}
		else
		{
			m_index2++;
		}
		if( m_index2 == (m_scaled_vec.length - 1) )
		{
			m_has_next = false;
		}
	}

	@Override
	public boolean hasNext() {
		return m_has_next;
	}

	@Override
	public IndividualVariables getIds() {
		return m_ids;
	}
	
	

}
